@@ -36,6 +36,7 @@
 #include "enums.h"
 #include "utils.h"
 #include "focusable.h"
+#include "focus-manager.h"
 
 /* Define this class in GObject system */
 static void _xfdashboard_viewpad_focusable_iface_init(XfdashboardFocusableInterface *iface);
@@ -224,6 +225,8 @@ static void _xfdashboard_viewpad_activate_view(XfdashboardViewpad *self, Xfdashb
 {
 	XfdashboardViewpadPrivate	*priv;
 	gfloat						x, y;
+	XfdashboardFocusManager		*focusManager;
+	gboolean					hasFocus;
 
 	g_return_if_fail(XFDASHBOARD_IS_VIEWPAD(self));
 	g_return_if_fail(inView==NULL || XFDASHBOARD_IS_VIEW(inView));
@@ -249,9 +252,20 @@ static void _xfdashboard_viewpad_activate_view(XfdashboardViewpad *self, Xfdashb
 		return;
 	}
 
+	/* Determine if this viewpad has the focus because we have to move focus in this case */
+	focusManager=xfdashboard_focus_manager_get_default();
+	hasFocus=xfdashboard_focus_manager_has_focus(focusManager, XFDASHBOARD_FOCUSABLE(self));
+
 	/* Deactivate current view */
 	if(priv->activeView)
 	{
+		/* Unset focus at current active view if this viewpad has the focus */
+		if(hasFocus)
+		{
+			xfdashboard_focusable_unset_focus(XFDASHBOARD_FOCUSABLE(priv->activeView));
+			g_debug("Viewpad has focus so unset focus from view '%s'", xfdashboard_view_get_name(priv->activeView));
+		}
+
 		/* Hide current view and emit signal before and after deactivation */
 		g_signal_emit(self, XfdashboardViewpadSignals[SIGNAL_VIEW_DEACTIVATING], 0, priv->activeView);
 		g_signal_emit_by_name(priv->activeView, "deactivating");
@@ -296,8 +310,34 @@ static void _xfdashboard_viewpad_activate_view(XfdashboardViewpad *self, Xfdashb
 
 		g_signal_emit_by_name(priv->activeView, "activated");
 		g_signal_emit(self, XfdashboardViewpadSignals[SIGNAL_VIEW_ACTIVATED], 0, priv->activeView);
+
+		/* Set focus to new active view if this viewpad has the focus */
+		if(hasFocus)
+		{
+			xfdashboard_focusable_set_focus(XFDASHBOARD_FOCUSABLE(priv->activeView));
+			g_debug("Viewpad has focus so set focus to view '%s'", xfdashboard_view_get_name(priv->activeView));
+		}
 	}
 
+	/* If no view is active at this time move focus to next focusable actor
+	 * if this viewpad has the focus.
+	 */
+	if(hasFocus && !priv->activeView)
+	{
+		XfdashboardFocusable	*newFocusable;
+
+		newFocusable=xfdashboard_focus_manager_get_next_focusable(focusManager, XFDASHBOARD_FOCUSABLE(self));
+		if(newFocusable)
+		{
+			xfdashboard_focus_manager_set_focus(focusManager, newFocusable);
+			g_debug("Viewpad has focus but no view is active so move focus to next focusable actor of type '%s'",
+					G_OBJECT_TYPE_NAME(newFocusable));
+		}
+	}
+
+	/* Release allocated resources */
+	if(focusManager) g_object_unref(focusManager);
+
 	/* Notify about property change */
 	g_object_notify_by_pspec(G_OBJECT(self), XfdashboardViewpadProperties[PROP_ACTIVE_VIEW]);
 }
@@ -959,9 +999,9 @@ static void _xfdashboard_viewpad_focusable_unset_focus(XfdashboardFocusable *inF
 	}
 }
 
-/* Virtual function "handle_key_event" was called */
-static gboolean _xfdashboard_viewpad_focusable_handle_key_event(XfdashboardFocusable *inFocusable,
-																const ClutterEvent *inEvent)
+/* Virtual function "handle_keypress_event" was called */
+static gboolean _xfdashboard_viewpad_focusable_handle_keypress_event(XfdashboardFocusable *inFocusable,
+																		const ClutterEvent *inEvent)
 {
 	XfdashboardViewpad			*self;
 	XfdashboardViewpadPrivate	*priv;
@@ -986,7 +1026,41 @@ static gboolean _xfdashboard_viewpad_focusable_handle_key_event(XfdashboardFocus
 	if(priv->activeView &&
 		XFDASHBOARD_IS_FOCUSABLE(priv->activeView))
 	{
-		handledEvent=xfdashboard_focusable_handle_key_event(XFDASHBOARD_FOCUSABLE(priv->activeView), inEvent);
+		handledEvent=xfdashboard_focusable_handle_keypress_event(XFDASHBOARD_FOCUSABLE(priv->activeView), inEvent);
+	}
+
+	/* Return focusable state */
+	return(handledEvent);
+}
+
+/* Virtual function "handle_keyrelease_event" was called */
+static gboolean _xfdashboard_viewpad_focusable_handle_keyrelease_event(XfdashboardFocusable *inFocusable,
+																		const ClutterEvent *inEvent)
+{
+	XfdashboardViewpad			*self;
+	XfdashboardViewpadPrivate	*priv;
+	gboolean					handledEvent;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable), CLUTTER_EVENT_PROPAGATE);
+	g_return_val_if_fail(XFDASHBOARD_IS_VIEWPAD(inFocusable), CLUTTER_EVENT_PROPAGATE);
+
+	self=XFDASHBOARD_VIEWPAD(inFocusable);
+	priv=self->priv;
+
+	/* Set handled key eventto CLUTTER_EVENT_PROPAGATE. It might be set to
+	 * CLUTTER_EVENT_STOP if current active view is focusable and it handled
+	 * the key event by its virtual function.
+	 */
+	handledEvent=CLUTTER_EVENT_PROPAGATE;
+
+	/* Viewpad is just a proxy for the current active view.
+	 * So check if current active view is focusable and call its
+	 * virtual function.
+	 */
+	if(priv->activeView &&
+		XFDASHBOARD_IS_FOCUSABLE(priv->activeView))
+	{
+		handledEvent=xfdashboard_focusable_handle_keyrelease_event(XFDASHBOARD_FOCUSABLE(priv->activeView), inEvent);
 	}
 
 	/* Return focusable state */
@@ -1001,7 +1075,8 @@ void _xfdashboard_viewpad_focusable_iface_init(XfdashboardFocusableInterface *if
 	iface->can_focus=_xfdashboard_viewpad_focusable_can_focus;
 	iface->set_focus=_xfdashboard_viewpad_focusable_set_focus;
 	iface->unset_focus=_xfdashboard_viewpad_focusable_unset_focus;
-	iface->handle_key_event=_xfdashboard_viewpad_focusable_handle_key_event;
+	iface->handle_keypress_event=_xfdashboard_viewpad_focusable_handle_keypress_event;
+	iface->handle_keyrelease_event=_xfdashboard_viewpad_focusable_handle_keyrelease_event;
 }
 
 /* IMPLEMENTATION: GObject */
