@@ -42,14 +42,17 @@
 
 #include "application.h"
 #include "marshal.h"
+#include "stylable.h"
 
 /* Define this class in GObject system */
-static void _xdashboard_window_content_clutter_content_iface_init(ClutterContentIface *inInterface);
+static void _xdashboard_window_content_clutter_content_iface_init(ClutterContentIface *iface);
+static void _xfdashboard_window_content_stylable_iface_init(XfdashboardStylableInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE(XfdashboardWindowContent,
 						xfdashboard_window_content,
 						G_TYPE_OBJECT,
-						G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_CONTENT, _xdashboard_window_content_clutter_content_iface_init))
+						G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_CONTENT, _xdashboard_window_content_clutter_content_iface_init)
+						G_IMPLEMENT_INTERFACE(XFDASHBOARD_TYPE_STYLABLE, _xfdashboard_window_content_stylable_iface_init))
 
 /* Private structure - access only by public API if needed */
 #define XFDASHBOARD_WINDOW_CONTENT_GET_PRIVATE(obj) \
@@ -62,6 +65,10 @@ struct _XfdashboardWindowContentPrivate
 	ClutterColor						*outlineColor;
 	gfloat								outlineWidth;
 	gboolean							isSuspended;
+	gboolean							includeWindowFrame;
+
+	gchar								*styleClasses;
+	gchar								*stylePseudoClasses;
 
 	/* Instance related */
 	gboolean							isFallback;
@@ -86,6 +93,11 @@ enum
 	PROP_SUSPENDED,
 	PROP_OUTLINE_COLOR,
 	PROP_OUTLINE_WIDTH,
+	PROP_INCLUDE_WINDOW_FRAME,
+
+	/* From interface: XfdashboardStylable */
+	PROP_STYLE_CLASSES,
+	PROP_STYLE_PSEUDO_CLASSES,
 
 	PROP_LAST
 };
@@ -511,6 +523,40 @@ static void _xfdashboard_window_content_resume(XfdashboardWindowContent *self)
 	g_debug("Resuming live texture updates for window '%s'", xfdashboard_window_tracker_window_get_title(priv->window));
 }
 
+/* Find X window for window frame of given X window content */
+static Window _xfdashboard_window_content_get_window_frame_xid(Display *inDisplay, Window inWindow)
+{
+	Window		iterXWindowID;
+	Window		rootXWindowID;
+	Window		foundXWindowID;
+
+	g_return_val_if_fail(inDisplay, 0);
+	g_return_val_if_fail(inWindow!=0, 0);
+
+	/* Iterate through X window tree list upwards until root window reached.
+	 * The last X window before root window is the one we are looking for.
+	 */
+	rootXWindowID=0;
+	foundXWindowID=0;
+	for(iterXWindowID=inWindow; iterXWindowID && iterXWindowID!=rootXWindowID; )
+	{
+		Window	*children;
+		guint	numberChildren;
+
+		children=NULL;
+		numberChildren=0;
+		foundXWindowID=iterXWindowID;
+		if(!XQueryTree(inDisplay, iterXWindowID, &rootXWindowID, &iterXWindowID, &children, &numberChildren))
+		{
+			iterXWindowID=0;
+		}
+		if(children) XFree(children);
+	}
+
+	/* Return found X window ID */
+	return(foundXWindowID);
+}
+
 /* Set window to handle and to display */
 static void _xfdashboard_window_content_set_window(XfdashboardWindowContent *self, XfdashboardWindowTrackerWindow *inWindow)
 {
@@ -522,6 +568,7 @@ static void _xfdashboard_window_content_set_window(XfdashboardWindowContent *sel
 	g_return_if_fail(XFDASHBOARD_IS_WINDOW_CONTENT(self));
 	g_return_if_fail(inWindow!=NULL && XFDASHBOARD_IS_WINDOW_TRACKER_WINDOW(inWindow));
 	g_return_if_fail(self->priv->window==NULL);
+	g_return_if_fail(self->priv->xWindowID==0);
 
 	priv=self->priv;
 
@@ -548,7 +595,16 @@ static void _xfdashboard_window_content_set_window(XfdashboardWindowContent *sel
 	priv->isFallback=TRUE;
 
 	/* Get X window and its attributes */
-	priv->xWindowID=xfdashboard_window_tracker_window_get_xid(priv->window);
+	if(priv->includeWindowFrame)
+	{
+		priv->xWindowID=_xfdashboard_window_content_get_window_frame_xid(display, xfdashboard_window_tracker_window_get_xid(priv->window));
+	}
+
+	if(!priv->xWindowID)
+	{
+		priv->xWindowID=xfdashboard_window_tracker_window_get_xid(priv->window);
+	}
+
 	if(!XGetWindowAttributes(display, priv->xWindowID, &windowAttrs))
 	{
 		g_warning(_("Could not get attributes of window '%s'"), xfdashboard_window_tracker_window_get_title(priv->window));
@@ -789,10 +845,58 @@ static gboolean _xdashboard_window_content_clutter_content_iface_get_preferred_s
 }
 
 /* Initialize interface of type ClutterContent */
-static void _xdashboard_window_content_clutter_content_iface_init(ClutterContentIface *inInterface)
+static void _xdashboard_window_content_clutter_content_iface_init(ClutterContentIface *iface)
 {
-	inInterface->get_preferred_size=_xdashboard_window_content_clutter_content_iface_get_preferred_size;
-	inInterface->paint_content=_xdashboard_window_content_clutter_content_iface_paint_content;
+	iface->get_preferred_size=_xdashboard_window_content_clutter_content_iface_get_preferred_size;
+	iface->paint_content=_xdashboard_window_content_clutter_content_iface_paint_content;
+}
+
+/* IMPLEMENTATION: Interface XfdashboardStylable */
+
+/* Get stylable properties of stage */
+static void _xfdashboard_window_content_stylable_get_stylable_properties(XfdashboardStylable *self,
+																			GHashTable *ioStylableProperties)
+{
+	g_return_if_fail(XFDASHBOARD_IS_STYLABLE(self));
+
+	/* Add stylable properties to hashtable */
+	xfdashboard_stylable_add_stylable_property(self, ioStylableProperties, "include-window-frame");
+}
+
+/* Get/set style classes of stage */
+static const gchar* _xfdashboard_window_content_stylable_get_classes(XfdashboardStylable *inStylable)
+{
+	/* Not implemented */
+	return(NULL);
+}
+
+static void _xfdashboard_window_content_stylable_set_classes(XfdashboardStylable *inStylable, const gchar *inStyleClasses)
+{
+	/* Not implemented */
+}
+
+/* Get/set style pseudo-classes of stage */
+static const gchar* _xfdashboard_window_content_stylable_get_pseudo_classes(XfdashboardStylable *inStylable)
+{
+	/* Not implemented */
+	return(NULL);
+}
+
+static void _xfdashboard_window_content_stylable_set_pseudo_classes(XfdashboardStylable *inStylable, const gchar *inStylePseudoClasses)
+{
+	/* Not implemented */
+}
+
+/* Interface initialization
+ * Set up default functions
+ */
+void _xfdashboard_window_content_stylable_iface_init(XfdashboardStylableInterface *iface)
+{
+	iface->get_stylable_properties=_xfdashboard_window_content_stylable_get_stylable_properties;
+	iface->get_classes=_xfdashboard_window_content_stylable_get_classes;
+	iface->set_classes=_xfdashboard_window_content_stylable_set_classes;
+	iface->get_pseudo_classes=_xfdashboard_window_content_stylable_get_pseudo_classes;
+	iface->set_pseudo_classes=_xfdashboard_window_content_stylable_set_pseudo_classes;
 }
 
 /* IMPLEMENTATION: GObject */
@@ -827,6 +931,18 @@ static void _xfdashboard_window_content_dispose(GObject *inObject)
 		priv->suspendSignalID=0;
 	}
 
+	if(priv->styleClasses)
+	{
+		g_free(priv->styleClasses);
+		priv->styleClasses=NULL;
+	}
+
+	if(priv->stylePseudoClasses)
+	{
+		g_free(priv->stylePseudoClasses);
+		priv->stylePseudoClasses=NULL;
+	}
+
 	/* Call parent's class dispose method */
 	G_OBJECT_CLASS(xfdashboard_window_content_parent_class)->dispose(inObject);
 }
@@ -837,7 +953,7 @@ static void _xfdashboard_window_content_set_property(GObject *inObject,
 												const GValue *inValue,
 												GParamSpec *inSpec)
 {
-	XfdashboardWindowContent	*self=XFDASHBOARD_WINDOW_CONTENT(inObject);
+	XfdashboardWindowContent		*self=XFDASHBOARD_WINDOW_CONTENT(inObject);
 
 	switch(inPropID)
 	{
@@ -853,6 +969,18 @@ static void _xfdashboard_window_content_set_property(GObject *inObject,
 			xfdashboard_window_content_set_outline_width(self, g_value_get_float(inValue));
 			break;
 
+		case PROP_INCLUDE_WINDOW_FRAME:
+			xfdashboard_window_content_set_include_window_frame(self, g_value_get_boolean(inValue));
+			break;
+
+		case PROP_STYLE_CLASSES:
+			_xfdashboard_window_content_stylable_set_classes(XFDASHBOARD_STYLABLE(self), g_value_get_string(inValue));
+			break;
+
+		case PROP_STYLE_PSEUDO_CLASSES:
+			_xfdashboard_window_content_stylable_set_pseudo_classes(XFDASHBOARD_STYLABLE(self), g_value_get_string(inValue));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
 			break;
@@ -864,8 +992,8 @@ static void _xfdashboard_window_content_get_property(GObject *inObject,
 												GValue *outValue,
 												GParamSpec *inSpec)
 {
-	XfdashboardWindowContent		*self=XFDASHBOARD_WINDOW_CONTENT(inObject);
-	XfdashboardWindowContentPrivate	*priv=self->priv;
+	XfdashboardWindowContent			*self=XFDASHBOARD_WINDOW_CONTENT(inObject);
+	XfdashboardWindowContentPrivate		*priv=self->priv;
 
 	switch(inPropID)
 	{
@@ -885,6 +1013,18 @@ static void _xfdashboard_window_content_get_property(GObject *inObject,
 			g_value_set_float(outValue, priv->outlineWidth);
 			break;
 
+		case PROP_INCLUDE_WINDOW_FRAME:
+			g_value_set_boolean(outValue, priv->includeWindowFrame);
+			break;
+
+		case PROP_STYLE_CLASSES:
+			g_value_set_string(outValue, priv->styleClasses);
+			break;
+
+		case PROP_STYLE_PSEUDO_CLASSES:
+			g_value_set_string(outValue, priv->stylePseudoClasses);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(inObject, inPropID, inSpec);
 			break;
@@ -897,13 +1037,17 @@ static void _xfdashboard_window_content_get_property(GObject *inObject,
  */
 void xfdashboard_window_content_class_init(XfdashboardWindowContentClass *klass)
 {
-	GObjectClass		*gobjectClass=G_OBJECT_CLASS(klass);
+	GObjectClass					*gobjectClass=G_OBJECT_CLASS(klass);
+	XfdashboardStylableInterface	*stylableIface;
+	GParamSpec						*paramSpec;
 
 	/* Override functions */
 	gobjectClass->dispose=_xfdashboard_window_content_dispose;
 	gobjectClass->set_property=_xfdashboard_window_content_set_property;
 	gobjectClass->get_property=_xfdashboard_window_content_get_property;
 
+	stylableIface=g_type_default_interface_ref(XFDASHBOARD_TYPE_STYLABLE);
+
 	/* Set up private structure */
 	g_type_class_add_private(klass, sizeof(XfdashboardWindowContentPrivate));
 
@@ -937,7 +1081,25 @@ void xfdashboard_window_content_class_init(XfdashboardWindowContentClass *klass)
 							1.0f,
 							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+	XfdashboardWindowContentProperties[PROP_INCLUDE_WINDOW_FRAME]=
+		g_param_spec_boolean("include-window-frame",
+							_("Include window frame"),
+							_("Whether the window frame should be included or only the window content should be shown"),
+							FALSE,
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+	paramSpec=g_object_interface_find_property(stylableIface, "style-classes");
+	XfdashboardWindowContentProperties[PROP_STYLE_CLASSES]=
+		g_param_spec_override("style-classes", paramSpec);
+
+	paramSpec=g_object_interface_find_property(stylableIface, "style-pseudo-classes");
+	XfdashboardWindowContentProperties[PROP_STYLE_PSEUDO_CLASSES]=
+		g_param_spec_override("style-pseudo-classes", paramSpec);
+
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardWindowContentProperties);
+
+	/* Release allocated resources */
+	g_type_default_interface_unref(stylableIface);
 }
 
 /* Object initialization
@@ -964,6 +1126,9 @@ void xfdashboard_window_content_init(XfdashboardWindowContent *self)
 	priv->isSuspended=TRUE;
 	priv->suspendSignalID=0;
 	priv->isMapped=FALSE;
+	priv->includeWindowFrame=FALSE;
+	priv->styleClasses=NULL;
+	priv->stylePseudoClasses=NULL;
 
 	/* Check extensions (will only be done once) */
 	_xfdashboard_window_content_check_extension();
@@ -971,6 +1136,9 @@ void xfdashboard_window_content_init(XfdashboardWindowContent *self)
 	/* Add event filter for this instance */
 	clutter_x11_add_filter(_xfdashboard_window_content_on_x_event, self);
 
+	/* Style content */
+	xfdashboard_stylable_invalidate(XFDASHBOARD_STYLABLE(self));
+
 	/* Handle suspension signals from application */
 	app=xfdashboard_application_get_default();
 	priv->suspendSignalID=g_signal_connect_swapped(app,
@@ -1100,3 +1268,53 @@ void xfdashboard_window_content_set_outline_width(XfdashboardWindowContent *self
 		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardWindowContentProperties[PROP_OUTLINE_WIDTH]);
 	}
 }
+
+/* Get/set flag to indicate whether to include the window frame or not */
+gboolean xfdashboard_window_content_get_include_window_frame(XfdashboardWindowContent *self)
+{
+	g_return_val_if_fail(XFDASHBOARD_IS_WINDOW_CONTENT(self), TRUE);
+
+	return(self->priv->includeWindowFrame);
+}
+
+void xfdashboard_window_content_set_include_window_frame(XfdashboardWindowContent *self, const gboolean inIncludeFrame)
+{
+	XfdashboardWindowContentPrivate				*priv;
+
+	g_return_if_fail(XFDASHBOARD_IS_WINDOW_CONTENT(self));
+
+	priv=self->priv;
+
+	/* Set value if changed */
+	if(priv->includeWindowFrame!=inIncludeFrame)
+	{
+		/* Set value */
+		priv->includeWindowFrame=inIncludeFrame;
+
+		/* (Re-)Setup window content */
+		if(priv->window)
+		{
+			XfdashboardWindowTrackerWindow		*window;
+
+			/* Re-setup window by releasing all resources first and unsetting window
+			 * but remember window to set it again.
+			 */
+			_xfdashboard_window_content_release_resources(self);
+
+			/* libwnck resources should never be freed. Just set to NULL */
+			window=priv->window;
+			priv->window=NULL;
+
+			/* Now set same window again which causes this object to set up all
+			 * needed X resources again.
+			 */
+			_xfdashboard_window_content_set_window(self, window);
+		}
+
+		/* Invalidate ourselve to get us redrawn */
+		clutter_content_invalidate(CLUTTER_CONTENT(self));
+
+		/* Notify about property change */
+		g_object_notify_by_pspec(G_OBJECT(self), XfdashboardWindowContentProperties[PROP_INCLUDE_WINDOW_FRAME]);
+	}
+}
